@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdal_mdreader.h 33694 2016-03-10 17:54:30Z goatbar $
+ * $Id: gdal_mdreader.h 5826aa92872e593c21b23965f998fbab1232be52 2022-01-14 17:23:09 +0100 Lunyxis $
  *
  * Project:  GDAL Core
  * Purpose:  Read metadata (mainly the remote sensing imagery) from files of
@@ -31,7 +31,6 @@
 #ifndef GDAL_MDREADER_H_INCLUDED
 #define GDAL_MDREADER_H_INCLUDED
 
-
 #include "cpl_port.h"
 #include "gdal_priv.h"
 
@@ -52,6 +51,8 @@
  * RPC/RPB specific defines
  */
 
+#define RPC_ERR_BIAS        "ERR_BIAS"
+#define RPC_ERR_RAND        "ERR_RAND"
 #define RPC_LINE_OFF        "LINE_OFF"
 #define RPC_SAMP_OFF        "SAMP_OFF"
 #define RPC_LAT_OFF         "LAT_OFF"
@@ -67,6 +68,18 @@
 #define RPC_SAMP_NUM_COEFF  "SAMP_NUM_COEFF"
 #define RPC_SAMP_DEN_COEFF  "SAMP_DEN_COEFF"
 
+/* Optional */
+#define RPC_MIN_LONG        "MIN_LONG"
+#define RPC_MIN_LAT         "MIN_LAT"
+#define RPC_MAX_LONG        "MAX_LONG"
+#define RPC_MAX_LAT         "MAX_LAT"
+
+/* Pleiades Neo nomenclature */
+#define RPC_LAT_NUM_COEFF  "LAT_NUM_COEFF"
+#define RPC_LAT_DEN_COEFF  "LAT_DEN_COEFF"
+#define RPC_LON_NUM_COEFF  "LON_NUM_COEFF"
+#define RPC_LON_DEN_COEFF  "LON_DEN_COEFF"
+
 /**
  * Enumerator of metadata readers
  */
@@ -88,11 +101,13 @@ typedef enum {
                MDR_LS | MDR_RE | MDR_KOMPSAT | MDR_EROS | MDR_ALOS /**< any reader */
 } MDReaders;
 
-
 /**
  * The base class for all metadata readers
  */
-class GDALMDReaderBase{
+class CPL_DLL GDALMDReaderBase{
+
+    CPL_DISALLOW_COPY_ASSIGN(GDALMDReaderBase)
+
 public:
     GDALMDReaderBase(const char *pszPath, char **papszSiblingFiles);
     virtual ~GDALMDReaderBase();
@@ -156,11 +171,13 @@ protected:
     virtual char** AddXMLNameValueToList(char** papszList, const char *pszName,
                                          const char *pszValue);
 protected:
-    char **m_papszIMDMD;
-    char **m_papszRPCMD;
-    char **m_papszIMAGERYMD;
-    char **m_papszDEFAULTMD;
-    bool m_bIsMetadataLoad;
+//! @cond Doxygen_Suppress
+    char **m_papszIMDMD = nullptr;
+    char **m_papszRPCMD = nullptr;
+    char **m_papszIMAGERYMD = nullptr;
+    char **m_papszDEFAULTMD = nullptr;
+    bool m_bIsMetadataLoad = false;
+//! @endcond
 };
 
 /**
@@ -169,6 +186,9 @@ protected:
  * for provided path.
  */
 class CPL_DLL GDALMDReaderManager{
+
+    CPL_DISALLOW_COPY_ASSIGN(GDALMDReaderManager)
+
 public:
     GDALMDReaderManager();
     virtual ~GDALMDReaderManager();
@@ -187,7 +207,9 @@ public:
                                         char **papszSiblingFiles,
                                         GUInt32 nType = MDR_ANY);
 protected:
-    GDALMDReaderBase *m_pReader;
+//! @cond Doxygen_Suppress
+    GDALMDReaderBase *m_pReader = nullptr;
+//! @endcond
 };
 
 // misc
