@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalwarper.h 27044 2014-03-16 23:41:27Z rouault $
+ * $Id: gdalwarper.h db9ea60631eda5ef881ae75dc3355f42734fb38c 2022-02-22 16:16:41 +0100 Even Rouault $
  *
  * Project:  GDAL High Performance Warper
  * Purpose:  Prototypes, and definitions for warping related work.
@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2003, Frank Warmerdam
- * Copyright (c) 2009-2012, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2009-2012, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -41,9 +41,11 @@
 
 #include "gdal_alg.h"
 #include "cpl_minixml.h"
+#include "cpl_multiproc.h"
 
 CPL_C_START
 
+/* Note: values are selected to be consistent with GDALRIOResampleAlg of gcore/gdal.h */
 /*! Warp Resampling Algorithm */
 typedef enum {
   /*! Nearest neighbour (select on one input pixel) */ GRA_NearestNeighbour=0,
@@ -51,46 +53,71 @@ typedef enum {
   /*! Cubic Convolution Approximation (4x4 kernel) */  GRA_Cubic=2,
   /*! Cubic B-Spline Approximation (4x4 kernel) */     GRA_CubicSpline=3,
   /*! Lanczos windowed sinc interpolation (6x6 kernel) */ GRA_Lanczos=4,
-  /*! Average (computes the average of all non-NODATA contributing pixels) */ GRA_Average=5, 
-  /*! Mode (selects the value which appears most often of all the sampled points) */ GRA_Mode=6
+  /*! Average (computes the weighted average of all non-NODATA contributing pixels) */ GRA_Average=5,
+  /*! Mode (selects the value which appears most often of all the sampled points) */ GRA_Mode=6,
+  /*  GRA_Gauss=7 reserved. */
+  /*! Max (selects maximum of all non-NODATA contributing pixels) */ GRA_Max=8,
+  /*! Min (selects minimum of all non-NODATA contributing pixels) */ GRA_Min=9,
+  /*! Med (selects median of all non-NODATA contributing pixels) */ GRA_Med=10,
+  /*! Q1 (selects first quartile of all non-NODATA contributing pixels) */ GRA_Q1=11,
+  /*! Q3 (selects third quartile of all non-NODATA contributing pixels) */ GRA_Q3=12,
+  /*! Sum (weighed sum of all non-NODATA contributing pixels). Added in GDAL 3.1 */ GRA_Sum=13,
+  /*! RMS (weighted root mean square (quadratic mean) of all non-NODATA contributing pixels) */ GRA_RMS=14,
+/*! @cond Doxygen_Suppress */
+  GRA_LAST_VALUE=GRA_RMS
+/*! @endcond */
 } GDALResampleAlg;
 
-typedef int 
+/*! GWKAverageOrMode Algorithm */
+typedef enum {
+    /*! Average */ GWKAOM_Average=1,
+    /*! Mode */ GWKAOM_Fmode=2,
+    /*! Mode of GDT_Byte, GDT_UInt16, or GDT_Int16 */ GWKAOM_Imode=3,
+    /*! Maximum */ GWKAOM_Max=4,
+    /*! Minimum */ GWKAOM_Min=5,
+    /*! Quantile */ GWKAOM_Quant=6,
+    /*! Sum */ GWKAOM_Sum=7,
+    /*! RMS */ GWKAOM_RMS=8
+} GWKAverageOrModeAlg;
+
+/*! @cond Doxygen_Suppress */
+typedef int
 (*GDALMaskFunc)( void *pMaskFuncArg,
-                 int nBandCount, GDALDataType eType, 
-                 int nXOff, int nYOff, 
+                 int nBandCount, GDALDataType eType,
+                 int nXOff, int nYOff,
                  int nXSize, int nYSize,
-                 GByte **papabyImageData, 
+                 GByte **papabyImageData,
                  int bMaskIsFloat, void *pMask );
 
-CPLErr CPL_DLL 
+CPLErr CPL_DLL
 GDALWarpNoDataMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
                       int nXOff, int nYOff, int nXSize, int nYSize,
                       GByte **papabyImageData, int bMaskIsFloat,
-                      void *pValidityMask );
+                      void *pValidityMask, int* pbOutAllValid );
 
-CPLErr CPL_DLL 
+CPLErr CPL_DLL
 GDALWarpDstAlphaMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
                         int nXOff, int nYOff, int nXSize, int nYSize,
                         GByte ** /*ppImageData */,
                         int bMaskIsFloat, void *pValidityMask );
-CPLErr CPL_DLL 
+CPLErr CPL_DLL
 GDALWarpSrcAlphaMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
                         int nXOff, int nYOff, int nXSize, int nYSize,
                         GByte ** /*ppImageData */,
-                        int bMaskIsFloat, void *pValidityMask );
+                        int bMaskIsFloat, void *pValidityMask, int* pbOutAllOpaque );
 
-CPLErr CPL_DLL 
+CPLErr CPL_DLL
 GDALWarpSrcMaskMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
                        int nXOff, int nYOff, int nXSize, int nYSize,
                        GByte ** /*ppImageData */,
                        int bMaskIsFloat, void *pValidityMask );
 
-CPLErr CPL_DLL 
+CPLErr CPL_DLL
 GDALWarpCutlineMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
                        int nXOff, int nYOff, int nXSize, int nYSize,
                        GByte ** /* ppImageData */,
                        int bMaskIsFloat, void *pValidityMask );
+/*! @endcond */
 
 /************************************************************************/
 /*                           GDALWarpOptions                            */
@@ -98,11 +125,11 @@ GDALWarpCutlineMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
 
 /** Warp control options for use with GDALWarpOperation::Initialize()  */
 typedef struct {
-    
-    char              **papszWarpOptions;  
+
+    char              **papszWarpOptions;
 
     /*! In bytes, 0.0 for internal default */
-    double              dfWarpMemoryLimit; 
+    double              dfWarpMemoryLimit;
 
     /*! Resampling algorithm to use */
     GDALResampleAlg     eResampleAlg;
@@ -112,14 +139,14 @@ typedef struct {
     GDALDataType        eWorkingDataType;
 
     /*! Source image dataset. */
-    GDALDatasetH	hSrcDS;
+    GDALDatasetH        hSrcDS;
 
     /*! Destination image dataset - may be NULL if only using GDALWarpOperation::WarpRegionToBuffer(). */
     GDALDatasetH        hDstDS;
 
     /*! Number of bands to process, may be 0 to select all bands. */
     int                 nBandCount;
-    
+
     /*! The band numbers for the source bands to process (1 based) */
     int                *panSrcBands;
 
@@ -134,14 +161,16 @@ typedef struct {
 
     /*! The "nodata" value real component for each input band, if NULL there isn't one */
     double             *padfSrcNoDataReal;
-    /*! The "nodata" value imaginary component - may be NULL even if real 
-      component is provided. */
+    /*! The "nodata" value imaginary component - may be NULL even if real
+      component is provided. This value is not used to flag invalid values.
+      Only the real component is used. */
     double             *padfSrcNoDataImag;
 
     /*! The "nodata" value real component for each output band, if NULL there isn't one */
     double             *padfDstNoDataReal;
-    /*! The "nodata" value imaginary component - may be NULL even if real 
-      component is provided. */
+    /*! The "nodata" value imaginary component - may be NULL even if real
+      component is provided. Note that warp operations only use real component
+      for flagging invalid data.*/
     double             *padfDstNoDataImag;
 
     /*! GDALProgressFunc() compatible progress reporting function, or NULL
@@ -157,25 +186,39 @@ typedef struct {
     /*! Handle to image transformer setup structure */
     void                *pTransformerArg;
 
+    /** Unused. Must be NULL */
     GDALMaskFunc       *papfnSrcPerBandValidityMaskFunc;
+    /** Unused. Must be NULL */
     void              **papSrcPerBandValidityMaskFuncArg;
-    
+
+    /** Unused. Must be NULL */
     GDALMaskFunc        pfnSrcValidityMaskFunc;
+    /** Unused. Must be NULL */
     void               *pSrcValidityMaskFuncArg;
-    
+
+    /** Unused. Must be NULL */
     GDALMaskFunc        pfnSrcDensityMaskFunc;
+    /** Unused. Must be NULL */
     void               *pSrcDensityMaskFuncArg;
 
+    /** Unused. Must be NULL */
     GDALMaskFunc        pfnDstDensityMaskFunc;
+    /** Unused. Must be NULL */
     void               *pDstDensityMaskFuncArg;
 
+    /** Unused. Must be NULL */
     GDALMaskFunc        pfnDstValidityMaskFunc;
+    /** Unused. Must be NULL */
     void               *pDstValidityMaskFuncArg;
 
+    /** Unused. Must be NULL */
     CPLErr              (*pfnPreWarpChunkProcessor)( void *pKern, void *pArg );
+    /** Unused. Must be NULL */
     void               *pPreWarpProcessorArg;
-    
+
+    /** Unused. Must be NULL */
     CPLErr              (*pfnPostWarpChunkProcessor)( void *pKern, void *pArg);
+    /** Unused. Must be NULL */
     void               *pPostWarpProcessorArg;
 
     /*! Optional OGRPolygonH for a masking cutline. */
@@ -191,30 +234,53 @@ void CPL_DLL CPL_STDCALL GDALDestroyWarpOptions( GDALWarpOptions * );
 GDALWarpOptions CPL_DLL * CPL_STDCALL
 GDALCloneWarpOptions( const GDALWarpOptions * );
 
+void CPL_DLL CPL_STDCALL
+GDALWarpInitDstNoDataReal( GDALWarpOptions *, double dNoDataReal );
+
+void CPL_DLL CPL_STDCALL
+GDALWarpInitSrcNoDataReal( GDALWarpOptions *, double dNoDataReal );
+
+void CPL_DLL CPL_STDCALL
+GDALWarpInitNoDataReal( GDALWarpOptions *, double dNoDataReal );
+
+void CPL_DLL CPL_STDCALL
+GDALWarpInitDstNoDataImag( GDALWarpOptions *, double dNoDataImag );
+
+void CPL_DLL CPL_STDCALL
+GDALWarpInitSrcNoDataImag( GDALWarpOptions *, double dNoDataImag );
+
+void CPL_DLL CPL_STDCALL
+GDALWarpResolveWorkingDataType( GDALWarpOptions * );
+
+void CPL_DLL CPL_STDCALL
+GDALWarpInitDefaultBandMapping( GDALWarpOptions *, int nBandCount );
+
+/*! @cond Doxygen_Suppress */
 CPLXMLNode CPL_DLL * CPL_STDCALL
       GDALSerializeWarpOptions( const GDALWarpOptions * );
 GDALWarpOptions CPL_DLL * CPL_STDCALL
       GDALDeserializeWarpOptions( CPLXMLNode * );
+/*! @endcond */
 
 /************************************************************************/
 /*                         GDALReprojectImage()                         */
 /************************************************************************/
 
 CPLErr CPL_DLL CPL_STDCALL
-GDALReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT, 
+GDALReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT,
                     GDALDatasetH hDstDS, const char *pszDstWKT,
                     GDALResampleAlg eResampleAlg, double dfWarpMemoryLimit,
                     double dfMaxError,
-                    GDALProgressFunc pfnProgress, void *pProgressArg, 
+                    GDALProgressFunc pfnProgress, void *pProgressArg,
                     GDALWarpOptions *psOptions );
 
 CPLErr CPL_DLL CPL_STDCALL
-GDALCreateAndReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT, 
+GDALCreateAndReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT,
                     const char *pszDstFilename, const char *pszDstWKT,
                     GDALDriverH hDstDriver, char **papszCreateOptions,
                     GDALResampleAlg eResampleAlg, double dfWarpMemoryLimit,
                     double dfMaxError,
-                    GDALProgressFunc pfnProgress, void *pProgressArg, 
+                    GDALProgressFunc pfnProgress, void *pProgressArg,
                     GDALWarpOptions *psOptions );
 
 /************************************************************************/
@@ -222,90 +288,150 @@ GDALCreateAndReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT,
 /************************************************************************/
 
 GDALDatasetH CPL_DLL CPL_STDCALL
-GDALAutoCreateWarpedVRT( GDALDatasetH hSrcDS, 
-                         const char *pszSrcWKT, const char *pszDstWKT, 
-                         GDALResampleAlg eResampleAlg, 
+GDALAutoCreateWarpedVRT( GDALDatasetH hSrcDS,
+                         const char *pszSrcWKT, const char *pszDstWKT,
+                         GDALResampleAlg eResampleAlg,
                          double dfMaxError, const GDALWarpOptions *psOptions );
 
-GDALDatasetH CPL_DLL CPL_STDCALL 
-GDALCreateWarpedVRT( GDALDatasetH hSrcDS, 
+GDALDatasetH CPL_DLL CPL_STDCALL
+GDALAutoCreateWarpedVRTEx( GDALDatasetH hSrcDS,
+                           const char *pszSrcWKT, const char *pszDstWKT,
+                           GDALResampleAlg eResampleAlg,
+                           double dfMaxError, const GDALWarpOptions *psOptions,
+                           CSLConstList papszTransformerOptions );
+
+GDALDatasetH CPL_DLL CPL_STDCALL
+GDALCreateWarpedVRT( GDALDatasetH hSrcDS,
                      int nPixels, int nLines, double *padfGeoTransform,
                      GDALWarpOptions *psOptions );
 
 CPLErr CPL_DLL CPL_STDCALL
-GDALInitializeWarpedVRT( GDALDatasetH hDS, 
+GDALInitializeWarpedVRT( GDALDatasetH hDS,
                          GDALWarpOptions *psWO );
 
 CPL_C_END
 
-#ifdef __cplusplus 
+#if defined(__cplusplus) && !defined(CPL_SUPRESS_CPLUSPLUS)
+
+#include <vector>
+#include <utility>
 
 /************************************************************************/
 /*                            GDALWarpKernel                            */
 /*                                                                      */
-/*      This class represents the lowest level of abstraction.  It      */
-/*      is holds the imagery for one "chunk" of a warp, and the         */
-/*      pre-prepared masks.  All IO is done before and after it's       */
-/*      operation.  This class is not normally used by the              */
-/*      application.                                                    */
-/************************************************************************/
 
-// This is the number of dummy pixels that must be reserved in source arrays
-// in order to satisfy assumptions made in GWKResample(), and more specifically
-// by GWKGetPixelRow() that always read a even number of pixels. So if we are
-// in the situation to read the last pixel of the source array, we need 1 extra
-// dummy pixel to avoid reading out of bounds.
+/** This is the number of dummy pixels that must be reserved in source arrays
+ * in order to satisfy assumptions made in GWKResample(), and more specifically
+ * by GWKGetPixelRow() that always read a even number of pixels. So if we are
+ * in the situation to read the last pixel of the source array, we need 1 extra
+ * dummy pixel to avoid reading out of bounds. */
 #define WARP_EXTRA_ELTS    1
 
+/** This class represents the lowest level of abstraction of warping.
+ *
+ * It holds the imagery for one "chunk" of a warp, and the
+ * pre-prepared masks.  All IO is done before and after its
+ * operation.  This class is not normally used by the
+ * application.
+ */
 class CPL_DLL GDALWarpKernel
 {
+    CPL_DISALLOW_COPY_ASSIGN(GDALWarpKernel)
+
 public:
+    /** Warp options */
     char              **papszWarpOptions;
 
-    GDALResampleAlg	eResample;
+    /** Resample algorithm */
+    GDALResampleAlg     eResample;
+    /** Working data type */
     GDALDataType        eWorkingDataType;
+    /** Number of input and output bands (excluding alpha bands) */
     int                 nBands;
 
+    /** Width of the source image */
     int                 nSrcXSize;
+    /** Height of the source image */
     int                 nSrcYSize;
-    GByte               **papabySrcImage; /* each subarray must have WARP_EXTRA_ELTS at the end */
+    /** Extra pixels (included in nSrcXSize) reserved for filter window. Should be ignored in scale computation */
+    double              dfSrcXExtraSize;
+    /** Extra pixels (included in nSrcYSize) reserved for filter window. Should be ignored in scale computation */
+    double              dfSrcYExtraSize;
+    /** Array of nBands source images of size nSrcXSize * nSrcYSize. Each subarray must have WARP_EXTRA_ELTS at the end */
+    GByte               **papabySrcImage;
 
-    GUInt32           **papanBandSrcValid; /* each subarray must have WARP_EXTRA_ELTS at the end */
-    GUInt32            *panUnifiedSrcValid; /* must have WARP_EXTRA_ELTS at the end */
-    float              *pafUnifiedSrcDensity; /* must have WARP_EXTRA_ELTS at the end */
+    /** Array of nBands validity mask of size (nSrcXSize * nSrcYSize + WARP_EXTRA_ELTS) / 8 */
+    GUInt32           **papanBandSrcValid;
+    /** Unified validity mask of size (nSrcXSize * nSrcYSize + WARP_EXTRA_ELTS) / 8 */
+    GUInt32            *panUnifiedSrcValid;
+    /** Unified source density of size nSrcXSize * nSrcYSize + WARP_EXTRA_ELTS */
+    float              *pafUnifiedSrcDensity;
 
+    /** Width of the destination image */
     int                 nDstXSize;
+    /** Height of the destination image */
     int                 nDstYSize;
+    /** Array of nBands destination images of size nDstXSize * nDstYSize */
     GByte             **papabyDstImage;
+    /** Validify mask of size (nDstXSize * nDstYSize) / 8 */
     GUInt32            *panDstValid;
+    /** Destination density of size nDstXSize * nDstYSize */
     float              *pafDstDensity;
 
-    double              dfXScale;   // Resampling scale, i.e.
-    double              dfYScale;   // nDstSize/nSrcSize.
-    double              dfXFilter;  // Size of filter kernel.
+    /** X resampling scale, i.e. nDstXSize / nSrcXSize */
+    double              dfXScale;
+    /** Y resampling scale, i.e. nDstYSize / nSrcYSize */
+    double              dfYScale;
+    /** X size of filter kernel */
+    double              dfXFilter;
+    /** Y size of filter kernel */
     double              dfYFilter;
-    int                 nXRadius;   // Size of window to filter.
+    /** X size of window to filter */
+    int                 nXRadius;
+    /** Y size of window to filter */
     int                 nYRadius;
-    int                 nFiltInitX; // Filtering offset
+    /** X filtering offset */
+    int                 nFiltInitX;
+    /** Y filtering offset */
     int                 nFiltInitY;
-    
+
+    /** X offset of the source buffer regarding the top-left corner of the image */
     int                 nSrcXOff;
+    /** Y offset of the source buffer regarding the top-left corner of the image */
     int                 nSrcYOff;
 
+    /** X offset of the destination buffer regarding the top-left corner of the image */
     int                 nDstXOff;
+    /** Y offset of the destination buffer regarding the top-left corner of the image */
     int                 nDstYOff;
-        
+
+    /** Pixel transformation function */
     GDALTransformerFunc pfnTransformer;
+    /** User data provided to pfnTransformer */
     void                *pTransformerArg;
 
+    /** Progress function */
     GDALProgressFunc    pfnProgress;
+    /** User data provided to pfnProgress */
     void                *pProgress;
 
+    /** Base/offset value for progress computation */
     double              dfProgressBase;
+    /** Scale value for progress computation */
     double              dfProgressScale;
-    
+
+    /** Array of nBands value for destination nodata */
     double              *padfDstNoDataReal;
 
+/*! @cond Doxygen_Suppress */
+    /** Per-thread data. Internally set */
+    void                *psThreadData;
+
+    bool                bApplyVerticalShift = false;
+
+    double              dfMultFactorVerticalShit = 1.0;
+/*! @endcond */
+
                        GDALWarpKernel();
     virtual           ~GDALWarpKernel();
 
@@ -313,6 +439,13 @@ public:
     CPLErr              PerformWarp();
 };
 
+/*! @cond Doxygen_Suppress */
+void* GWKThreadsCreate(char** papszWarpOptions,
+                       GDALTransformerFunc pfnTransformer,
+                       void* pTransformerArg);
+void GWKThreadsEnd(void* psThreadDataIn);
+/*! @endcond */
+
 /************************************************************************/
 /*                         GDALWarpOperation()                          */
 /*                                                                      */
@@ -323,69 +456,109 @@ public:
 /*      masks.  Actual resampling is done by the GDALWarpKernel.        */
 /************************************************************************/
 
+/*! @cond Doxygen_Suppress */
+typedef struct _GDALWarpChunk GDALWarpChunk;
+/*! @endcond */
+
 class CPL_DLL GDALWarpOperation {
+
+    CPL_DISALLOW_COPY_ASSIGN(GDALWarpOperation)
+
 private:
     GDALWarpOptions *psOptions;
 
     void            WipeOptions();
     int             ValidateOptions();
 
-    CPLErr          ComputeSourceWindow( int nDstXOff, int nDstYOff, 
+    CPLErr          ComputeSourceWindow( int nDstXOff, int nDstYOff,
                                          int nDstXSize, int nDstYSize,
-                                         int *pnSrcXOff, int *pnSrcYOff, 
-                                         int *pnSrcXSize, int *pnSrcYSize );
+                                         int *pnSrcXOff, int *pnSrcYOff,
+                                         int *pnSrcXSize, int *pnSrcYSize,
+                                         double *pdfSrcXExtraSize, double *pdfSrcYExtraSize,
+                                         double* pdfSrcFillRatio );
 
-    CPLErr          CreateKernelMask( GDALWarpKernel *, int iBand, 
+    void            ComputeSourceWindowStartingFromSource(
+                                    int nDstXOff, int nDstYOff,
+                                    int nDstXSize, int nDstYSize,
+                                    double* padfSrcMinX, double* padfSrcMinY,
+                                    double* padfSrcMaxX, double* padfSrcMaxY);
+
+    static CPLErr          CreateKernelMask( GDALWarpKernel *, int iBand,
                                       const char *pszType );
 
-    void            *unused1;
-    void            *unused2;
-    void            *hIOMutex;
-    void            *hWarpMutex;
+    CPLMutex        *hIOMutex;
+    CPLMutex        *hWarpMutex;
 
     int             nChunkListCount;
     int             nChunkListMax;
-    int            *panChunkList;
+    GDALWarpChunk  *pasChunkList;
 
     int             bReportTimings;
     unsigned long   nLastTimeReported;
 
+    void           *psThreadData;
+
+    // Coordinates a few special points in target image space, to determine
+    // if ComputeSourceWindow() must use a grid based sampling.
+    std::vector<std::pair<double, double>> aDstXYSpecialPoints{};
+
+    bool m_bIsTranslationOnPixelBoundaries = false;
+
     void            WipeChunkList();
-    CPLErr          CollectChunkList( int nDstXOff, int nDstYOff, 
+    CPLErr          CollectChunkListInternal( int nDstXOff, int nDstYOff,
+                                      int nDstXSize, int nDstYSize );
+    void            CollectChunkList( int nDstXOff, int nDstYOff,
                                       int nDstXSize, int nDstYSize );
     void            ReportTiming( const char * );
-    
+
 public:
                     GDALWarpOperation();
     virtual        ~GDALWarpOperation();
 
     CPLErr          Initialize( const GDALWarpOptions *psNewOptions );
+    void*           CreateDestinationBuffer( int nDstXSize, int nDstYSize,
+                                             int *pbWasInitialized = nullptr );
+    static void     DestroyDestinationBuffer(void* pDstBuffer);
 
     const GDALWarpOptions         *GetOptions();
 
-    CPLErr          ChunkAndWarpImage( int nDstXOff, int nDstYOff, 
+    CPLErr          ChunkAndWarpImage( int nDstXOff, int nDstYOff,
                                        int nDstXSize, int nDstYSize );
-    CPLErr          ChunkAndWarpMulti( int nDstXOff, int nDstYOff, 
+    CPLErr          ChunkAndWarpMulti( int nDstXOff, int nDstYOff,
                                        int nDstXSize, int nDstYSize );
-    CPLErr          WarpRegion( int nDstXOff, int nDstYOff, 
+    CPLErr          WarpRegion( int nDstXOff, int nDstYOff,
                                 int nDstXSize, int nDstYSize,
                                 int nSrcXOff=0, int nSrcYOff=0,
                                 int nSrcXSize=0, int nSrcYSize=0,
                                 double dfProgressBase=0.0, double dfProgressScale=1.0);
-    
-    CPLErr          WarpRegionToBuffer( int nDstXOff, int nDstYOff, 
-                                        int nDstXSize, int nDstYSize, 
-                                        void *pDataBuf, 
+    CPLErr          WarpRegion( int nDstXOff, int nDstYOff,
+                                int nDstXSize, int nDstYSize,
+                                int nSrcXOff, int nSrcYOff,
+                                int nSrcXSize, int nSrcYSize,
+                                double dfSrcXExtraSize, double dfSrcYExtraSize,
+                                double dfProgressBase, double dfProgressScale);
+    CPLErr          WarpRegionToBuffer( int nDstXOff, int nDstYOff,
+                                        int nDstXSize, int nDstYSize,
+                                        void *pDataBuf,
                                         GDALDataType eBufDataType,
                                         int nSrcXOff=0, int nSrcYOff=0,
                                         int nSrcXSize=0, int nSrcYSize=0,
                                         double dfProgressBase=0.0, double dfProgressScale=1.0);
+    CPLErr          WarpRegionToBuffer( int nDstXOff, int nDstYOff,
+                                        int nDstXSize, int nDstYSize,
+                                        void *pDataBuf,
+                                        GDALDataType eBufDataType,
+                                        int nSrcXOff, int nSrcYOff,
+                                        int nSrcXSize, int nSrcYSize,
+                                        double dfSrcXExtraSize, double dfSrcYExtraSize,
+                                        double dfProgressBase, double dfProgressScale);
 };
 
 #endif /* def __cplusplus */
 
 CPL_C_START
 
+/** Opaque type representing a GDALWarpOperation object */
 typedef void * GDALWarpOperationH;
 
 GDALWarpOperationH CPL_DLL GDALCreateWarpOperation(const GDALWarpOptions* );
@@ -398,6 +571,21 @@ CPLErr CPL_DLL GDALWarpRegionToBuffer( GDALWarpOperationH, int, int, int, int,
                                        void *, GDALDataType,
                                        int, int, int, int );
 
+/************************************************************************/
+/*      Warping kernel functions                                        */
+/************************************************************************/
+
+/*! @cond Doxygen_Suppress */
+int GWKGetFilterRadius(GDALResampleAlg eResampleAlg);
+
+typedef double (*FilterFuncType)(double dfX);
+FilterFuncType GWKGetFilterFunc(GDALResampleAlg eResampleAlg);
+
+// TODO(schwehr): Can padfVals be a const pointer?
+typedef double (*FilterFunc4ValuesType)(double* padfVals);
+FilterFunc4ValuesType GWKGetFilterFunc4Values(GDALResampleAlg eResampleAlg);
+/*! @endcond */
+
 CPL_C_END
 
 #endif /* ndef GDAL_ALG_H_INCLUDED */
