@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_feature.h eb925c89fcc7978ce21d7456480f31487bcb8ad1 2021-04-04 09:37:15 +0200 Even Rouault $
+ * $Id: ogr_feature.h 1e4510d0d88bbf73885b7f18b79f50d5a6696131 2021-08-21 19:26:01 +0200 Even Rouault $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Class for representing a whole feature, and layer schemas.
@@ -279,17 +279,14 @@ class CPL_DLL OGRFeatureDefn
 {
   protected:
 //! @cond Doxygen_Suppress
-    volatile int nRefCount;
+    volatile int nRefCount = 0;
 
-    mutable int         nFieldCount;
-    mutable OGRFieldDefn **papoFieldDefn;
+    mutable std::vector<std::unique_ptr<OGRFieldDefn>> apoFieldDefn{};
+    mutable std::vector<std::unique_ptr<OGRGeomFieldDefn>> apoGeomFieldDefn{};
 
-    mutable int                nGeomFieldCount;
-    mutable OGRGeomFieldDefn **papoGeomFieldDefn;
+    char        *pszFeatureClassName = nullptr;
 
-    char        *pszFeatureClassName;
-
-    int         bIgnoreStyle;
+    bool        bIgnoreStyle = false;
 //! @endcond
 
   public:
@@ -305,17 +302,17 @@ class CPL_DLL OGRFeatureDefn
     virtual int         GetFieldIndex( const char * ) const;
     int                 GetFieldIndexCaseSensitive( const char * ) const;
 
-    virtual void        AddFieldDefn( OGRFieldDefn * );
+    virtual void        AddFieldDefn( const OGRFieldDefn * );
     virtual OGRErr      DeleteFieldDefn( int iField );
-    virtual OGRErr      ReorderFieldDefns( int* panMap );
+    virtual OGRErr      ReorderFieldDefns( const int* panMap );
 
     virtual int         GetGeomFieldCount() const;
     virtual OGRGeomFieldDefn *GetGeomFieldDefn( int i );
     virtual const OGRGeomFieldDefn *GetGeomFieldDefn( int i ) const;
     virtual int         GetGeomFieldIndex( const char * ) const;
 
-    virtual void        AddGeomFieldDefn( OGRGeomFieldDefn *,
-                                          int bCopy = TRUE );
+    virtual void        AddGeomFieldDefn( const OGRGeomFieldDefn * );
+    virtual void        AddGeomFieldDefn( std::unique_ptr<OGRGeomFieldDefn>&& );
     virtual OGRErr      DeleteGeomFieldDefn( int iGeomField );
 
     virtual OGRwkbGeometryType GetGeomType() const;
@@ -330,8 +327,8 @@ class CPL_DLL OGRFeatureDefn
 
     virtual int         IsGeometryIgnored() const;
     virtual void        SetGeometryIgnored( int bIgnore );
-    virtual int         IsStyleIgnored() const { return bIgnoreStyle; }
-    virtual void        SetStyleIgnored( int bIgnore )
+    virtual bool        IsStyleIgnored() const { return bIgnoreStyle; }
+    virtual void        SetStyleIgnored( bool bIgnore )
         { bIgnoreStyle = bIgnore; }
 
     virtual int         IsSame( const OGRFeatureDefn * poOtherFeatureDefn ) const;
